/*
 * Copyright (c) 2002, ERIK WISTRAND
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 * - Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimer. 
 * 
 * - Redistributions in binary form must reproduce the above copyright notice, 
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution. 
 * 
 * - Neither the name of the VALENTINE MIDLET PROJECT nor the names of its 
 *   contributors may be used to endorse or promote products derived 
 *   from this software without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.wistrand.midlet.sprite;

import javax.microedition.lcdui.Graphics;

import java.util.Vector;
import com.wistrand.midlet.fp.FP;

/**
 * Sprite which uses a point array to draw a polygon.
 */
public abstract class PolySprite extends Sprite {


  public PolySprite() {
    super();
    size = 20 * FP.SCALE;
  }
  
  /**
   * Get the (x,y) points describing the sprite outline.
   */
  public abstract int[] getPoints();


  public void paint(Graphics g) {

    int[] points = getPoints();

    int lastX = -33333;  // marker for undefined point ;)
    int lastY = -1;

    int sin = FP.sin(rot);
    int cos = FP.cos(rot);
    
    for(int i = 0; i < points.length; i += 2) {
      int x0 = size * points[i];
      int y0 = size * points[i + 1];

      // Rotate around (0,0)
      int x1 = (x0 * cos - y0 * sin) >> FP.SCALE_BITS;
      int y1 = (y0 * cos + x0 * sin) >> FP.SCALE_BITS;

      // move it to current pos and scale
      int sx = (x + x1) >> FP.SCALE_BITS;
      int sy = (y + y1) >> FP.SCALE_BITS;

      if(lastX != -33333) {
	g.setColor(col);
	g.drawLine(lastX, lastY, sx, sy);
      }
      lastX = sx;
      lastY = sy;
    }
  }
}

