/*
 * Copyright (c) 2002, ERIK WISTRAND
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 * - Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimer. 
 * 
 * - Redistributions in binary form must reproduce the above copyright notice, 
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution. 
 * 
 * - Neither the name of the VALENTINE MIDLET PROJECT nor the names of its 
 *   contributors may be used to endorse or promote products derived 
 *   from this software without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.wistrand.midlet.sprite;

import javax.microedition.lcdui.*;
import javax.microedition.midlet.*;

import java.util.*;
import com.wistrand.midlet.fp.*;

/**
 * Base class for simple movable objects, sprites.
 */
public abstract class Sprite {

  public int x;
  public int y;

  public int dx;
  public int dy;
  public int size    = FP.SCALE;

  public int    rot  = 0;
  public int    col  = 0x000000;
  
  public int    id;
  public Sprite parent;
  public int    px;
  public int    py;
  
  protected static int idCount = 0;

  public Sprite() {
    id = idCount++;
  }

  /**
   * Set the sprite parent. Position will be relative to this parent.
   *
   * @param parent Sprite to be relative to. If <tt>null</tt>, don't use
   *               a parent.
   * @param px     X relative coordinate to parent.
   * @param py     Y relative coordinate to parent.
   */
  public void setParent(Sprite parent, int px, int py) {
    this.parent = parent;
    this.px     = px;
    this.py     = py;
    moveTo(x, y);
  }

  public void setColor(int col) {
    this.col = col;
  }

  /**
   * Move to a new coordinate.
   */
  public void moveTo(int x, int y) {
    if(parent != null) {
      this.x = px + parent.x;
      this.y = py + parent.y;
    } else {
      this.x = x;
      this.y = y;
    }
  }

  /**
   * Set sprite speed.
   */
  public void setSpeed(int dx, int dy) {
    this.dx = dx;
    this.dy = dy;
  }


  /**
   * Update position by adding speed to current pos.
   */
  public void update() {
    moveTo(x + dx, y + dy);
  }

  /**
   * Paint the sprite on a Graphics. Should be overridden
   * by subclasses since the default method does nothing.
   */
  public abstract void paint(Graphics g);

  public String toString() {
    return getClass().getName() + "[id=" + id + "]";
  }
}

