/*
 * Copyright (c) 2002, ERIK WISTRAND
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 * - Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimer. 
 * 
 * - Redistributions in binary form must reproduce the above copyright notice, 
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution. 
 * 
 * - Neither the name of the VALENTINE MIDLET PROJECT nor the names of its 
 *   contributors may be used to endorse or promote products derived 
 *   from this software without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.wistrand.midlet.valentine;

import javax.microedition.lcdui.*;
import javax.microedition.midlet.*;

import java.util.*;

import com.wistrand.midlet.sprite.*;
import com.wistrand.midlet.fp.*;

/**
 * Sprite whhich will balloon upward on the screen while showing a 
 * text.
 */
public class BalloonSprite extends Sprite {

  String        txt;
  SpriteHandler handler;
  int           darkCol;

  static Font font = Font.getFont(Font.FACE_PROPORTIONAL,
				  Font.STYLE_PLAIN,
				  Font.SIZE_SMALL);

  /**
   * Creat a new text balloon.
   *
   * @param handler The handler's <tt>remove</tt> method will be called
   *                when the ballon has reached the top of the screen.
   * @param x initial x position
   * @param y initial y position
   */
  public BalloonSprite(SpriteHandler handler, String txt, int x, int y) {
    super();
    this.txt     = txt;
    this.handler = handler;

    moveTo(x, y);

    if(Main.theMain.canvas.bgColor == 0xffffff) {
      setColor(0x000000);
    } else {
      setColor(0xffffff);
    }
  }

  public void setColor(int col) {
    super.setColor(col);
    if(col == 0x000000) {
      darkCol = -1;
    } else {
      darkCol = Util.interpolateRGB(0x000000, col, 128);
    }
  }

  /**
   * Set the text of the balloon.
   */
  public void setText(String txt) {
    this.txt = txt;
  }

  public void update() {
    setSpeed(FP.rnd(-2 * FP.SCALE, 2 * FP.SCALE),
	     FP.rnd(-4 * FP.SCALE, FP.SCALE / 2));
    super.update();

    int border = (txt.length() * 3) << FP.SCALE_BITS;
    int w = Main.theMain.canvas.getWidth()  << FP.SCALE_BITS;
    if(x < border)     { x = border; }
    if(x > w - border) { x = w - border; }

    if(y <= 0) {
      handler.remove(this);
    }
  }

  public void paint(Graphics g) {

    int sx = x / FP.SCALE;
    int sy = y / FP.SCALE;

    g.setFont(font);

    int ix = 0;
    int start = 0;
    while(-1 != (ix = txt.indexOf("\n", start))) {
      draw(g, txt, start, ix, sx, sy);
      start = ix + 1;
      sy += font.getHeight();
    }
    draw(g, txt, start, txt.length(), sx, sy);
  }

  void draw(Graphics g, String txt, int start, int ix, int sx, int sy) {
    if(darkCol != -1) {
      g.setColor(darkCol);
      g.drawSubstring(txt, start, ix - start, sx-1, sy-1, Graphics.TOP|Graphics.HCENTER);
    }

    g.setColor(col);
    g.drawSubstring(txt, start, ix - start, sx, sy, Graphics.TOP|Graphics.HCENTER);
  }

  public String toString() {
    return "BalloonSprite[id=" + id + ", txt=" + txt + "]";
  }
}

