/*
 * Copyright (c) 2002, ERIK WISTRAND
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 * - Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimer. 
 * 
 * - Redistributions in binary form must reproduce the above copyright notice, 
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution. 
 * 
 * - Neither the name of the VALENTINE MIDLET PROJECT nor the names of its 
 *   contributors may be used to endorse or promote products derived 
 *   from this software without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.wistrand.midlet.valentine;

import javax.microedition.lcdui.*;
import javax.microedition.midlet.*;
import javax.microedition.rms.*;

import com.wistrand.midlet.sprite.*;
import com.wistrand.midlet.fp.*;

public class Main extends MIDlet implements CommandListener, KeyListener {
  
  SpriteCanvas canvas;

  private Command exitCommand   = new Command("No love", Command.EXIT,   99);

  private Command addCommand    = new Command("Love",    Command.SCREEN, 1);
  private Command runCommand    = new Command("Play",    Command.ITEM, 1);
  private Command setupCommand  = new Command("Setup",   Command.BACK, 2);

  Display display;

  public static Main theMain;

  Form      welcomeForm;
  Form      setupForm;
  TextField userName;

  RecordStore rs;

  String name = "";

  ChoiceGroup colors;
  String      color = Util.colors[0];

  static int ID_NAME  = 1;
  static int ID_COLOR = 2;

  static String RS_NAME = "info";

  public Main() {


    theMain = this;

    display = Display.getDisplay(this);

    loadDb();

    if(color.equals("")) color = Util.colors[0];

    boolean bAgain = name.length() > 0;

    welcomeForm = new Form("Hi");
    setupForm   = new Form("Setup");

    userName = new TextField(bAgain ? "Is that you again?" : "It's nice with a name?", name, 18, TextField.ANY);

    setupForm.append(userName);

    colors = new ChoiceGroup("Background", ChoiceGroup.EXCLUSIVE);
    
    for(int i = 0; i < Util.colors.length; i += 2) {
      colors.append(Util.colors[i], null);
      if(Util.colors[i].equals(color)) {
	colors.setSelectedIndex(i / 2, true);
      }
    }

    setupForm.append(colors);
    setupForm.addCommand(runCommand);
    setupForm.setCommandListener(this);


    if(!bAgain) {
      Util.setName(Util.makeName());
    }
    welcomeForm.append(new StringItem(bAgain ? "Is that you?" 
				      : Util.makeText(), name));

    welcomeForm.addCommand(runCommand);
    welcomeForm.addCommand(setupCommand);
    welcomeForm.setCommandListener(this);


    canvas = new SpriteCanvas(display, this);
    
    canvas.setCommandListener(this);
    canvas.setBgColor(0xffffff);

    canvas.addCommand(exitCommand);
    canvas.addCommand(addCommand);
    
    for(int i = 0; i < 1; i++) {
      addHeart();
    }
  }

  void openDb() {
    try {
      if(rs == null) {
	rs = RecordStore.openRecordStore(RS_NAME, true);
      }
      if(rs.getNumRecords() != 2) {
	byte[] buf = "".getBytes();
	
	rs.closeRecordStore();
	RecordStore.deleteRecordStore(RS_NAME);	
	rs = RecordStore.openRecordStore(RS_NAME, true);
	
	rs.addRecord(buf, 0, buf.length);
	rs.addRecord(buf, 0, buf.length);
      }
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  void loadDb() {
    openDb();
    try {
      name  = new String(rs.getRecord(ID_NAME));
      color = new String(rs.getRecord(ID_COLOR));
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  void saveDb() {
    openDb();
    try {
      byte [] buf0 = name.getBytes();
      byte [] buf1 = color.getBytes();

      rs.setRecord(ID_NAME,  buf0, 0, buf0.length);
      rs.setRecord(ID_COLOR, buf1, 0, buf1.length);

    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  void addHeart() {
    int x = canvas.getWidth() / 2;
    int y = canvas.getHeight() / 2;
    canvas.add(new HeartSprite(canvas, x, y));
  }

  public void commandAction(Command c, Displayable s) {
    if (c == runCommand) {
      name      = userName.getString();
      int colIx = colors.getSelectedIndex();
      if(colIx != -1) {
	color = Util.colors[colIx * 2];
	canvas.setBgColor(Util.getColor(color));
      }
      String info = "";
      if(name.length() == 0) {
	name = Util.makeName();
	info = "I will call you " + name;
      } else {
	info = "Hi " + name;
      }
      Util.setName(name);
      saveDb();
      startHeart();
      Sprite sp = new BalloonSprite(canvas,
				    info,
				    (canvas.getWidth() / 2) << FP.SCALE_BITS,
				    (canvas.getHeight() / 2) << FP.SCALE_BITS);
      canvas.add(sp);
    } else if (c == setupCommand) {
      Display.getDisplay(this).setCurrent(setupForm);
    } else if (c == addCommand) {
      addHeart();
    } else if (c == exitCommand) {
      try {
	destroyApp(false);
	notifyDestroyed();
      } catch (Exception ex) {
      }
    }
  }

  void startHeart() {
    Display.getDisplay(this).setCurrent(canvas);
    canvas.start();
  }

  public void startApp() {
    Display.getDisplay(this).setCurrent(welcomeForm);
  }

  public void pauseApp()   {
    canvas.stop();
  }
    
  public void destroyApp(boolean unconditional)  {
    saveDb();
    canvas.stop();
  }
 
  void exitRequested() {
    destroyApp(false);
    notifyDestroyed();
  }



  public void keyPressed(int keyCode, int gameAction) {
    switch (gameAction) {
    case Canvas.UP:
      {
	Sprite s0 = (Sprite)canvas.sprites.elementAt(FP.rnd(0, canvas.sprites.size()));
	Sprite s = new MultiStar(canvas.bgColor, canvas);
	s.setParent(s0, 0, 0);
	canvas.addBefore(s, s0);
      }
      break;    
    case Canvas.DOWN:
      {
	Sprite s0 = (Sprite)canvas.sprites.elementAt(FP.rnd(0, canvas.sprites.size()));
	if(s0 instanceof BalloonSprite) {
	  ((BalloonSprite)s0).setText(Util.makeText());
	} else {
	  Sprite s = new BalloonSprite(canvas, Util.makeText(), s0.x, s0.y); 
	  canvas.add(s);
	}
      }
      break;
    case Canvas.LEFT:
      {
	canvas.clearSprites();
	addHeart();
      }
      break;
    case Canvas.RIGHT:
      // togglePause();
      break;
    case Canvas.FIRE:
      break;
    default:
      break;
    }
    switch (keyCode) {
    case Canvas.KEY_NUM5: 
      canvas.bUseBgImage = false;
      canvas.bgImage     = null;
      canvas.bgGraphics  = null;
      color = Util.nextColor();
      canvas.bgColor = Util.getColor(color);
      break;
    case Canvas.KEY_NUM7:
      canvas.bUseBgImage = !canvas.bUseBgImage;
      break;
    case Canvas.KEY_NUM1:
      canvas.nShowInfo = (canvas.nShowInfo + 1) % (canvas.MAXINFO + 1);
      break;
    default:
      break;
    }
  }
}


