/*
 * Copyright (c) 2002, ERIK WISTRAND
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 * - Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimer. 
 * 
 * - Redistributions in binary form must reproduce the above copyright notice, 
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution. 
 * 
 * - Neither the name of the VALENTINE MIDLET PROJECT nor the names of its 
 *   contributors may be used to endorse or promote products derived 
 *   from this software without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.wistrand.midlet.valentine;

import java.util.*;

import com.wistrand.midlet.fp.*;

public class Util {

  public static int interpolateRGB(int c0, int c1, int level) {
    int r0 = (c0 >> 16) & 0xff; 
    int g0 = (c0 >> 8 ) & 0xff; 
    int b0 = (c0)       & 0xff; 

    int r1 = (c1 >> 16) & 0xff; 
    int g1 = (c1 >> 8 ) & 0xff; 
    int b1 = (c1)       & 0xff; 

    int r = (r0 + ((level * (r1 - r0)) >> 8)) & 0xff;
    int g = (g0 + ((level * (g1 - g0)) >> 8)) & 0xff;
    int b = (b0 + ((level * (b1 - b0)) >> 8)) & 0xff;

    return (r << 16) | (g << 8) | b;
  }


  static String[] verbs = {
    "hug", "love", "kiss", "like", "adore", "miss", 
  };

  static String[] likeverbs = {
    "love", "like", 
  };

  static String[] verbs2 = {
    "think", "hope",
  };

  static String[] nouns1 = {
    "me", "I",
  };

  static String[] nouns2 = {
    "[name]", "you", "my friends"
  };

  static String[] nouns3 = {
    "me", "me", "your friend", 
  };

  static String[] nouns4 = {
    "[name]", "me", "your friend", 
  };

  static String[] endings = {
    " please", "?", ""
  };

  static String[] fixed = {
    "be my valentine",
    "me can help you",
    "you can hug me",
    "there is love",
    "follow me",
    
  };

  public static void setName(String s) {
    nouns2[0] = s;
    nouns4[0] = s;
  }

  /**
   * Make a nice text which sounds quite like an english sentence.
   */
  public static String makeText() {
    int type = Math.abs(FP.rnd.nextInt() % 7);
    StringBuffer sb = new StringBuffer();

    Calendar cal = Calendar.getInstance();
    Date     date = new Date();
    cal.setTime(date);	

    int hour    = cal.get(Calendar.HOUR_OF_DAY);
    int weekday = cal.get(Calendar.DAY_OF_WEEK);

    //    type = 4;
    switch(type) {
    case 0: 
      {
	String n1 = select(nouns1,  null);
	String v  = select(verbs,  null);  
	String n2 = select(nouns2, n1);
	sb.append(n1).append(" ").append(v).append(" ").append(n2);
      }
      break;
    case 1:
      {
	String v  = select(verbs, "miss");  
	String n  = select(nouns3, null);
	String e  = select(endings, null);
	sb.append(v).append(" ").append(n);
	
	append(sb, e);
      } 
      break;
    case 2:
      {
	String v  = select(verbs, "miss");  
	String e  = select(endings, null);
	sb.append(v);
	append(sb, e);
      }
      break;
    case 3:
      {
	String n1 = select(nouns1, null);
	String v  = select(verbs2, null);
	String n2 = select(nouns2, null);
	String v2  = select(verbs, v);  
	String n3 = select(nouns4, n2);
	sb.append(n1).append(" ").append(v).append(" ").append(n2).
	  append("\n").append(v2).append(" ").append(n3);
      }
      break;
    case 4:
      {

	if(hour < 7) {
	  sb.append(select(early1, null));
	} else if(hour < 10) {
	  sb.append(select(early2, null));
	} else if(hour < 13) {
	  sb.append(select(noon, null));
	} else if(hour < 15) {
	  sb.append(select(afternoon, null));
	} else if(hour < 20) {
	  sb.append(select(evening, null));
	} else if(hour < 24) {
	  sb.append(select(night, null));
	}
	sb.append(" now");
	append(sb, select(endings, null));
      }
      break;
    case 5:
      {
	int t2 = Math.abs(FP.rnd.nextInt() % fixed.length);
	sb.append(fixed[t2]);
	String e = select(endings, null);
	append(sb, e);
      }
      break;
    case 6:
      {
	sb.
	  append(select(nouns1, null)).
	  append(" ").
	  append(select(likeverbs, null)).
	  append(" ").
	  append(dayName(weekday));
      }
      break;
    default:
      sb.append("No alternative " + type);
    }

    return sb.toString();
  }

  static boolean isWhite(char c) {
    return -1 != " \n\t\r".indexOf(c);
  }

  static StringBuffer append(StringBuffer sb, String s) {
    int ix = sb.length() - 1;
    while(ix >= 0 && isWhite(sb.charAt(ix))) {
      ix--;
    }
    if(sb.length() - ix < 16) {
      sb.append(s);
    }
    return sb;
  }

  static String[] early1     = {"very early", "oups - morning" };
  static String[] early2     = {"early", "breakfast", "bright light" };
  static String[] noon       = {"lunch", "nice day" };
  static String[] afternoon  = {"tea", "snack" };
  static String[] evening    = {"nice evening", "hungry", "dinner" };
  static String[] night      = {"dark", "kind of late", "time for bed" };


  static String dayName(int n) {
    switch(n) {
    case Calendar.MONDAY:    return "monday";
    case Calendar.TUESDAY:   return "tuesday";
    case Calendar.WEDNESDAY: return "wednesday";
    case Calendar.THURSDAY:  return "thursday";
    case Calendar.FRIDAY:    return "friday";
    case Calendar.SATURDAY:  return "saturday";
    case Calendar.SUNDAY:    return "sunday";
    }
    return "no day " + n;
  }

  static String select(String[] sl, String not) {
    int r;
    do { 
      r = Math.abs(FP.rnd.nextInt()) % sl.length;
    } while(sl[r].equals(not));
    
    return sl[r];
  }

  static String[] names = {
    "cutiepie", "cutie", "darling", "love", "angel", "sunshine", "precious" 
  };

  static public String makeName() {
    return select(names, null);
  }

  static String[] colors = {
    "white",   "ffffff",
    "yellow",  "ffff00",
    "orange",  "ff7700",
    "red",     "ff0000",
    "black",   "000000",
    "blue",    "0000ff",
    "gray",    "777777",
    "green",   "00aa00",
  };

  static int currColor = 0;

  static String nextColor() {
    currColor = (currColor + 2) % colors.length;
    return colors[currColor];
  }

  static int getColor(String s) {
    for(int i = 0; i < colors.length; i += 2) {
      if(s.equals(colors[i])) {
	currColor = i;
	return Integer.parseInt(colors[i + 1], 16);
      }
    }
    return 0x000000;
  }

  /*
  static void main(String[] argv) {
    String name = makeName();
    System.out.println("hi " + name);
    setName(name);
    int n = 10;
    try {
      n = Integer.parseInt(argv[0]);
    } catch (Exception e) {  }
    for(int i = 0; i < n; i++) {
      System.out.println(makeText());
    }
  }
  */
}
