/*
 * Copyright (c) 2002, ERIK WISTRAND
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 * - Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimer. 
 * 
 * - Redistributions in binary form must reproduce the above copyright notice, 
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution. 
 * 
 * - Neither the name of the VALENTINE MIDLET PROJECT nor the names of its 
 *   contributors may be used to endorse or promote products derived 
 *   from this software without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.wistrand.midlet.valentine;

import javax.microedition.lcdui.*;
import javax.microedition.midlet.*;

import java.util.*;

import com.wistrand.midlet.sprite.*;
import com.wistrand.midlet.fp.*;


/**
 * Wobbler class
 */
public class Wobbler {

  int max     = FP.SCALE;
  int delta   = 0;
  int wobble  = 0;
  int decay   = 240;

  public Wobbler(int max, int decay) {
    this.max   = max;
    this.decay = decay;
  }
  
  public void update() {
    if(!isActive()) {
      return;
    }

    delta = (decay * delta) >> FP.SCALE_BITS;

    if(delta == 0) {
      wobble = 0;
    } else {
      wobble += delta;    
      if(wobble > max ) { wobble =  max; delta = -delta; }
      if(wobble < -max) { wobble = -max; delta = -delta; }
    }
  }

  public void start(int delta) {
    this.delta = delta;
  } 

  public void stop() {
    delta = 0;
  }

  public boolean isActive() {
    return (delta != 0);
  }

  public int getValue() {
    return isActive() 
      ? ((Math.abs(delta) * wobble) >> FP.SCALE_BITS)
      : 0;
  }
}

